unit GLUI;

interface

uses
    Classes,
    Windows,
    Controls,
    GL,
    GLTexture,
    GLText,
    Camera;

type
    TUIElement=class;
    TUIClickEvent=procedure (Sender:TUIElement) of object;
    TUIElement=class
                    private
                    //Top,Left, Width and Height of the element
                    fX,fY,fWidth,fHeight:integer;
                    //Is the mouse over it?
                    fHighlighted:boolean;
                    //Frame Update stuff
                    fFrameInterval,fLastTickCount:cardinal;
                    //Enabled/Disabled
                    fEnabled:boolean;
                    //Click Event
                    fOnClick:TUIClickEvent;
                    //The name of the object
                    fName:string;
                    public
                    constructor Create;
                    //Desscendants can override Update to perform
                    //Time keyed animation or whatever
                    function Update(TickCount:cardinal):boolean;virtual;
                    //All descendants MUST render themselves
                    procedure Render;virtual;abstract;
                    //Call this if you want to simulate
                    //a click on the button
                    procedure DoClick;virtual;
                    //DUH!!
                    property Name:string
                             read fName
                             write fName;
                    property X:integer
                             read fX
                             write fX;
                    property Y:integer
                             read fY
                             write fY;
                    property Width:integer
                             read fWidth
                             write fWidth;
                    property Height:integer
                             read fHeight
                             write fHeight;
                    property Highlighted:boolean
                             read fHighLighted
                             write fHighlighted;
                    property Enabled:boolean
                             read fEnabled
                             write fEnabled;
                    property OnClick:TUIClickEvent
                             read fOnClick
                             write fOnClick;
    end;
    TRollOverImage=class(TUIElement)
                        private
                        //The texture that is not highlighted
                        fBackTex,
                        //The highlight texture
                        fForeTex:TTexture;
                        public
                        constructor Create;
                        destructor Destroy;override;
                        procedure Render;override;
                        property BackTex:TTexture
                                 read fBackTex
                                 write fBackTex;
                        property ForeTex:TTexture
                                 read fForeTex
                                 write fForeTex;
    end;
    TRolloverButton=class(TRolloverImage)
                         private
                         fDisabledTex:TTexture;
                         public
                         constructor Create;
                         destructor Destroy;override;
                         procedure Render;override;
                         property DisabledTex:TTexture
                                  read fDisabledTex
                                  write fDisabledTex;
    end;
    TRollOverCheckBox=class(TRollOverButton)
                           private
                           fChecked:boolean;
                           fCheckedTex:TTexture;
                           public
                           constructor Create;
                           destructor Destroy;override;
                           procedure Render;override;
                           procedure DoClick;override;
                           property CheckedTex:TTexture
                                    read fCheckedTex
                                    write fCheckedTex;
    end;

    TUIManager=class
                    private
                    fElements:TList;
                    public
                    constructor Create;
                    destructor Destroy;override;
                    procedure MouseMove(X,Y:integer);
                    procedure MouseClick(X,Y:integer;Button:TMouseButton);
                    procedure Render;
                    property Elements:TList
                             read fElements
                             write fElements;
    end;

function PointInRect(mX,mY,X,Y,Width,Height:integer):boolean;

implementation

function PointInRect(mX,mY,X,Y,Width,Height:integer):boolean;
begin
     result:=(mX>X) and (mY>Y) and (mX<(X+Width)) and (mY<(Y+Height));
end;

{ TUIElement }

constructor TUIElement.Create;
begin
     fEnabled:=true;
end;

procedure TUIElement.DoClick;
begin
     //If the click event handler is assigned,
     //then it is invoked 
     if assigned(fOnClick) then fOnClick(Self);
end;

function TUIElement.Update(TickCount: cardinal): boolean;
begin
     //Set result to false initially
     result:=false;
     //If the object is disabled, then dont update.
     //This means successors will also not Update since
     //all of them should check inherited result before
     //proceeding with their Update
     if not fEnabled then exit;
     if (TickCount-fLastTickCount)>=fFrameInterval then
     begin
          //fFrameInterval ticks have passed since the lase Update
          //Updation can now take place
          fLastTickCount:=TickCount;
          result:=true;
     end;
end;

{ TRollOverImage }

constructor TRollOverImage.Create;
begin
     inherited;
     fForeTex:=TTexture.Create;
     fBackTex:=TTexture.Create;
end;

destructor TRollOverImage.Destroy;
begin
     inherited;
     fForeTex.Free;
     fBackTex.Free;
end;

procedure TRollOverImage.Render;
begin
     fBackTex.Enable;
     fBackTex.Opaque;
     DrawQuad(fX,fY,fWidth,fHeight);

     if fHighlighted then
     begin
          fForeTex.Enable;
          fForeTex.Transparency;
          DrawQuad(fX,fY,fWidth,fHeight);
     end;
end;

{ TRolloverButton }

constructor TRolloverButton.Create;
begin
     inherited;
     fDisabledTex:=TTexture.Create;
end;

destructor TRolloverButton.Destroy;
begin
     fDisabledTex.Free;
end;

procedure TRolloverButton.Render;
begin
     if Enabled then
        inherited Render
     else
     begin
          fDisabledTex.Enable;
          fDisabledTex.Opaque;
          DrawQuad(X,Y,Width,Height);
     end;
end;

{ TRollOverCheckBox }

constructor TRollOverCheckBox.Create;
begin
     inherited;
     fCheckedTex:=TTexture.Create;
end;

destructor TRollOverCheckBox.Destroy;
begin
     fCheckedTex.Free;
     inherited;
end;

procedure TRollOverCheckBox.DoClick;
begin
     inherited;
     fChecked:=not fChecked;
end;

procedure TRollOverCheckBox.Render;
begin
     inherited;
      if fChecked then
      begin
           fCheckedTex.Enable;
           fCheckedTex.Transparency;
           DrawQuad(fX,fY,fWidth,fHeight);
      end;
end;

{ TUIManager }

constructor TUIManager.Create;
begin
     fElements:=TList.Create;
end;

destructor TUIManager.Destroy;
var
   i:integer;
begin
     inherited;
     for i:=0 to fElements.Count-1 do
         TUIElement(fElements[i]).Free;
end;

procedure TUIManager.MouseClick(X, Y: integer; Button: TMouseButton);
var
   i:integer;
begin
     for i:=0 to fElements.Count-1 do
     if TUIElement(fElements[i]).Enabled then
        if PointInRect(X,Y,TUIElement(fElements[i]).X,TUIElement(fElements[i]).Y,TUIElement(fElements[i]).Width,TUIElement(fElements[i]).Height) then
           TUIElement(fElements[i]).DoClick;
end;

procedure TUIManager.MouseMove(X, Y: integer);
var
   i:integer;
begin
     for i:=0 to fElements.Count-1 do
     if TUIElement(fElements[i]).Enabled then
        if PointInRect(X,Y,TUIElement(fElements[i]).X,TUIElement(fElements[i]).Y,TUIElement(fElements[i]).Width,TUIElement(fElements[i]).Height) then
           TUIElement(fElements[i]).Highlighted:=true
        else
            TUIElement(fElements[i]).Highlighted:=false;
end;

procedure TUIManager.Render;
var
   i:integer;
begin
     for i:=0 to fElements.Count-1 do
         TUIElement(fElements[i]).Render;
end;

end.
